package com.shigongli.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.shigongli.common.*;
import com.shigongli.entity.TaShopKeeper;
import com.shigongli.service.ITaShopKeeperService;
import com.shigongli.vo.AuthPhoneParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import me.chanjar.weixin.mp.api.WxMpService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.shigongli.service.ITaPersonService;
import com.shigongli.entity.TaPerson;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 人员表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-12-11
 */

@Api(tags = "人员表")
@RestController
@RequestMapping("/")
public class TaPersonController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaPersonController.class);

    @Autowired
    public ITaPersonService iTaPersonService;

    @Autowired
    private ITaShopKeeperService iTaShopKeeperService;

    @Autowired
    private WxUtils wxUtils;

    @ApiOperation(value="授权手机", notes = "授权手机", httpMethod = "POST", response = ResponseBean.class)
    @PostMapping("/ma/taPerson/phone")
    public ResponseBean phone(@ApiParam("授权手机参数") @RequestBody AuthPhoneParams authPhoneParams,
                              HttpServletRequest request) {
        if (null == authPhoneParams) {
            return ResponseBean.error("请设置授权手机号的参数", ResponseBean.ERROR_UNAVAILABLE);
        }
        TaPerson taPerson = getPerson(request);
        if (null == taPerson) {
            return ResponseBean.error("校验用户失败, 请退出重新进入", ResponseBean.ERROR_UNAVAILABLE);
        }

        WxMaService maService = wxUtils.getMaService();
        WxMaPhoneNumberInfo phoneNoInfo = maService.getUserService().getPhoneNoInfo(authPhoneParams.getSessionKey(), authPhoneParams.getEncryptedData(), authPhoneParams.getIv());

        String phone = phoneNoInfo.getPhoneNumber();
        taPerson.setPhone(phone);
        iTaPersonService.updateById(taPerson);

        // 绑定店主
        iTaShopKeeperService.bindPerson(taPerson);

        // 关联的店主一起返回
        List<TaShopKeeper> shopKeeperList = iTaShopKeeperService.getByPerson(taPerson.getPersonId());
        if (null != shopKeeperList && shopKeeperList.size() > 0) {
            taPerson.setShopKeeperList(shopKeeperList);
        }

        return ResponseBean.success(taPerson);
    }


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taPerson",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taPersonList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                     @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     HttpServletRequest request) throws Exception{

		    IPage<TaPerson> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaPerson> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaPerson> result = iTaPersonService.page(pg, queryWrapper);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taPerson 实体对象
     * @return
     */
    @RequestMapping(value="/taPerson",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taPersonAdd(@ApiParam("保存内容") @RequestBody TaPerson taPerson) throws Exception{

        if (iTaPersonService.save(taPerson)){
            return ResponseBean.success(taPerson);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taPerson/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taPersonDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iTaPersonService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taPerson 实体对象
     * @return
     */
    @RequestMapping(value="/taPerson/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taPersonUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody TaPerson taPerson) throws Exception{

        if (iTaPersonService.updateById(taPerson)){
            return ResponseBean.success(iTaPersonService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taPerson/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taPersonGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaPersonService.getById(id));
    }
}
